/*;  ostring.h {\documentstyle[fleqn]{jsrc2latex}\font\tenrm=cmtt10} */

/*;  $$Revision$$ {\rhead{\sl ostring.h\quad\thepage}} */
/*;  $$Date$$  */
/*;  $$Source$$  */
/*;  $$RCShost$$  */

// @(#)  ostring.h -- string class

/* <japanese>
{\ {\setlength{\parskip}{4pt}% latex-mode
ʸ󥯥饹 C++׼˾, pp. 153--161.
}% end of latex-mode }
</japanese>
*/

#if !defined(_ostring_h)
#define _ostring_h

#include <stdio.h>
#include <string.h>
#include <iostream.h>

/*----------------------------------------------------------------------
CLASS
    OHyMosString

    string class

DESCRIPTION

    simple string class

    <japanese>
    ñ string class Ǥ
    </japanese>

----------------------------------------------------------------------*/

class OHyMosString {
private:
  char *s;
public:
  //////////////////////////////////////////////////////////////////////
  // Constructor
  OHyMosString(void);				// OHyMosString s;

  //////////////////////////////////////////////////////////////////////
  // Conversion constructor
  OHyMosString(const char*);		// OHyMosString s = "abcd";

  //////////////////////////////////////////////////////////////////////
  // Copy constructor
  OHyMosString(const OHyMosString&);		// OHyMosString p = s;

  //////////////////////////////////////////////////////////////////////
  // Destructor
  ~OHyMosString() { delete [] s; }

  //////////////////////////////////////////////////////////////////////
  // Assignment operator
  OHyMosString& operator=(const char*);

  //////////////////////////////////////////////////////////////////////
  // Assignment operator
  OHyMosString& operator=(const OHyMosString&);

  //////////////////////////////////////////////////////////////////////
  // Subscripting
  char& operator[](size_t);

  //////////////////////////////////////////////////////////////////////
  // Conversion operator to char*.  Note that a new area is created and
  // the pointer to that area is returned.
  operator char*();

  //////////////////////////////////////////////////////////////////////
  // Returns the length
  int Size() const { return strlen(s); }

  //////////////////////////////////////////////////////////////////////
  // Concatenates a string
  void operator += (const OHyMosString & str);

  //////////////////////////////////////////////////////////////////////
  // Concatenates a string
  void operator += (char* cstr);

  //////////////////////////////////////////////////////////////////////
  // Concatenates a character
  void operator += (char ch);

  //////////////////////////////////////////////////////////////////////
  // Comparison operator ==
  friend int operator==(const OHyMosString& x, const OHyMosString& y)
 		{ return (strcmp(x.s, y.s) == 0); }

  //////////////////////////////////////////////////////////////////////
  // Comparison operator !=
  friend int operator!=(const OHyMosString& x, const OHyMosString& y)
 		{ return (strcmp(x.s, y.s) != 0); }

  // stream input/output methods

 friend ostream & operator << (ostream & strm, const OHyMosString & str);
 friend istream & operator >> (istream & strm, OHyMosString & str);

  // <japanese>
  // operator >> λͤѹޤ
  // ϥȥ꡼θ֤߰ʸŤʸɤ߹ߡ ʸ(
  // 򡤥֡ʸ)ɤ߼ä顤ޤǤʸ KhString ˳Ǽ
  // ޤ줬ܡ
  // ϥȥ꡼फǽɤ߼äʸ, (󥰥륯
  // ޤϡ֥륯)ΤȤˤϡĤƱΰɤ߼
  // ޤǤʸ KhString ˳Ǽޤϡȥ꡼˻Ĥ
  // ɤ߼äʸˤϡޤޤ󡣰
  // ȤȤˤʤޤ
  // äơϢ³ư椬񤤤Ƥȡ̥ʸɤ߼ޤ
  // ǸĹʸѤ˻Ѥޤ
  // ɤ߼ʤäˤϡistream  failbit 򥻥åȤޤ
  // </japanese>
};

// stream I/O functions
inline ostream & operator << (ostream & strm, const OHyMosString & str)
{
  if (strlen(str.s) > 0)
    strm << str.s;
  return strm;
}

void dstr2data(char*& dstr, OHyMosString& a);

void data2file(OHyMosString& a, FILE* fp);

void data2ofile(OHyMosString& a, FILE* fo);

#endif // ! _ostring_h

/* end of file {\ % The following part will be omitted in tex file
\begin{comment}\begin{verbatim}
This part is not included in the tex output.
$Revision: 1.2 $

# Local Variables: ***
# tab-width: 4 ***
# End: ***
\end{verbatim}\end{comment}
} */


/*; Local Variables: *** */
/*; tab-width: 4 *** */
/*; fill-column: 72 *** */
/*; End: *** */

/*; end of file */
