/*;  receive.h {\documentstyle[fleqn]{jsrc2latex}\font\tenrm=cmtt10} */

/*;  $$Revision: 1.2 $$ {\rhead{\sl receive.h\quad\thepage}} */
/*;  $$Date: 2000/09/03 05:02:11 $$  */
/*;  $$Source: /home/shiiba/ohymos-lib/ohymos-current/receive.h,v $$  */
/*;  $$RCShost: yodo.kuciv.kyoto-u.ac.jp $$  */

// @(#) receive.h -- base class for creating receive ports

#if !defined(receive_h)
#define receive_h

#include <stdio.h>
#include <time.h>       //; time_t , tm 

#include "common.h"     //; ץˤ붦
#include "port.h"       //; 饹 Port 
#include "sender.h"     //; definition of class Sender_port


/*----------------------------------------------------------------
CLASS
    Receive_port

    Class for models of receive ports.

KEYWORDS

    receive, port

DESCRIPTION
    Class for models of receive ports. Derived from the class Port.

    The class Receive_port has a member for storing the pointer to a
    sender which sends datapacks to the current receive port.

    Member functions for

    <ul>
    <li>  connecting the current receive port to a corresponding sender
    <li>  receiving datapacks from the sender
    </ul>

    are defined.

    NOTE:

    Only one send port can be connected to an object of class
    Receive_port.

    <japanese>
    ܷüҥǥ \verb|Receive_port| ޤüҤˤ
    üҤĤĤʤȤǤޤ

    饹 Receive_port Ǥ,
    <pre>
        ǡѥåü    : sender
    </pre>
    򵭲,
    </pre>
        üҤȤ³ǽ
        sender Υǡѥåǽ
    </pre>
    ޤ.

    Receive_port ³ǤüҤϰĤǤ.
    </japanese>

----------------------------------------------------------------*/

class Receive_port : public Port {

//; 0.  ǡ.
protected:
  //////////////////////////////////////////////////////////////////////
  // Pointer to a sender which sends datapacks to the current object
  // <japanese>
  // void* ˥㥹Ȥ줿üҤΥɥ쥹
  // </japanese>
  void* sender;

//; 1.  ³ǽ.
public:
  //////////////////////////////////////////////////////////////////////
  // Returns a string which expresses the type of datapacks received by
  // the current object.  This is a pure virtual function.  This
  // function must be defined in derived classes.
  // <japanese>
  // ǡѥåɽʸ֤
  //
  // 쥯饹 Port ν貾۴ؿǤ.
  // Receive_port Ǥ貾۴ؿȤƤޤ.
  // Receive_port 饹, ǡѥåɽʸؤΥ
  // 󥿤֤褦ɬƲ.
  //
  // 㤨, ǡѥå "INT" Ǥ饹Ǥ,
  // <pre>
  //   char* Data_pack_type(void) { return "INT"; }
  // </pre>
  // Τ褦ޤ.
  //
  // ³ˤ, üҤüҤȤ Data_pack_type פ뤳
  // Ȥǧޤ.
  // <pre>
  // RETURN:
  //   char*      ǡѥåɽʸؤΥݥ.
  // </pre>
  // </japanese>
  virtual char* Data_pack_type(void) = NULL;

public:
  //////////////////////////////////////////////////////////////////////
  // Virtual function declared in the base class Port.  Set "sender"
  // to "s" and return YES.
  //
  // When one of the following cases occurs
  // <ul>
  // <li> s is NULL.
  // <li> "sender" has been set already.
  // <li> Data_pack_types between the current object and the sender
  //      are different
  // </ul>
  // this function prints an error message and returns NO.
  //
  // <japanese>
  // üҤ
  //
  // 쥯饹 Port ν貾۴ؿǤ.
  // ü sender  s ꤷ, YES ֤ޤ.
  //
  // ξϥ顼åɽ, NO ֤ޤ.
  // <pre>
  //  s  NULL ξ.
  //  üҤü sender Ѥߤξ.
  //  üҤ s , Data_pack_type ۤʤ.
  // </pre>
  // <pre>
  // ARGUMENT:
  //   s     void* ˥㥹Ȥ줿üҤΥɥ쥹.
  // RETURN:
  //   YES   ü sender ꤵ줿.
  //   NO    s  NULL ξ.
  //           üҤü sender Ѥߤξ.
  //           üҤ s , Data_pack_type ۤʤ.
  // ACCIDENT:
  //   s  Port* ѿǼƤʤ.
  // </pre>
  // </japanese>
  Boolean Set_sender(void* s);

public:
  //////////////////////////////////////////////////////////////////////
  // Virtual function declared in the base class Port.  Because no
  // corresponding work is needed in the Receive_port class, Switch of
  // the Receive_port class simply returns YES.
  //
  // Note: Switch of the Sender_port class is defined differently.
  //
  // <japanese>
  // ³ȸν
  //
  // 쥯饹 Port β۴ؿǤ.
  // Receive_port Ǥ, ³ȸԤ٤äˤޤ.
  //  YES ֤ޤ.
  //
  // <pre>
  //      RETURN:
  //        YES    YES ֤.
  // </pre>
  // </japanese>
  Boolean Switch(void) { return YES; }

public:
  //////////////////////////////////////////////////////////////////////
  // Answers whether the current receive port has been connected to some
  // sender or not.
  //
  // <japanese>
  // ׻γǧ
  //
  // 쥯饹 Port ν貾۴ؿǤ.
  // ǡѥåǽʾ֤Ǥ뤳Ȥǧޤ.
  // sender ꤵƤ, sender ǡѥåǤޤ.
  //
  // <pre>
  // RETURN:
  // YES   sender Ѥߤξ.
  // NO    sender ̤ξ.
  // </pre>
  // </japanese>
  Boolean Are_you_ready(void) { return (sender != NULL) ? YES : NO; }


//; 2.  ǡѥåμǽ, õǽ.
public:
  //////////////////////////////////////////////////////////////////////
  // Orders the sender to create a data_pack_array.
  //
  // <japanese>
  // ǡݥѥåκ
  //
  // 쥯饹 Port ν貾۴ؿǤ.
  // sender ˥ǡѥå褦̿ᤷޤ.
  // </japanese>
  void Set_data_pack_array(void)
        { ((Port*) sender)->Set_data_pack_array(); }

public:
  //////////////////////////////////////////////////////////////////////
  // Returns the data time of the oldest data pack in the data stored
  // in the sender.
  time_t Oldest_data_time(void)
        { return ((Port*) sender)->Oldest_data_time(); }

  //////////////////////////////////////////////////////////////////////
  // Returns the data time of the newest data pack in the data stored
  // in the sender.
    time_t Newest_data_time(void)
        { return ((Port*) sender)->Newest_data_time(); }

  //////////////////////////////////////////////////////////////////////
  // Returns the number of datapacks stored in the sender.
  int Number_of_data_packs(void)
    { return ((Port*) sender)->Number_of_data_packs(); }

  //////////////////////////////////////////////////////////////////////
  // Tells whether the data from "t_from" to "t_to" can be obtained or
  // not
  Boolean Can_you_get_data(time_t t_from, time_t t_to)
        { return ((Port*) sender)->Can_you_get_data(t_from, t_to); }

  //////////////////////////////////////////////////////////////////////
  // Tells whether the data at "t_fromto" can be obtained or not
  Boolean Can_you_get_data(time_t t_fromto)
        { return ((Port*) sender)->Can_you_get_data(t_fromto, t_fromto); }
        /* <japanese>
          Receive_port::Can_you_get_data(time_t,time_t) (virtual,inline)
                ǡѥå˴ؤǧ

        쥯饹 Port ν貾۴ؿǤ.
        sender 䤤碌, t_from  t_to ʹߤޤǤΥǡѥ
        ǽǧޤ.
        ǽʤ YES ֤, Բǽʤ NO ֤ޤ.

        μüҤ owner Ƿ׻γǧȤѤޤ.

        ARGUMENT:
            t_from    t_from ǡѥåǽǤ뤳Ȥ
                        ǧ.
            t_to      t_to ʹߤޤǥǡѥåǽǤ뤳Ȥ
                        ǧ.
        RETURN:
            YES       t_from  t_to ʹߤޤǤΥǡѥå
                        ǽʾ.
            NO        t_from  t_to ʹߤޤǤΥǡѥå
                        Բǽʾ.
        ACCIDENT:
            ³, sender  NULL ξ.
              δؿ, ׻ˤѤǽΤ, 
               sender  NULL ǤʤȤǧޤǤ.
            sender  Port* ʳѿǼƤ.
        </japanese> */
public:
  //////////////////////////////////////////////////////////////////////
  // Get a data pack array from the sender. After the execution of this
  // function, "dp_array" stores the data pack array and "num_of_dp"
  // stores the dimention of "dp_array".
  void Get_data_pack_array(void**& dp_array, int& num_of_dp)
        { ((Port*) sender)->Get_data_pack_array(dp_array, num_of_dp); }
        /* <japanese>
          Receive_port::Get_data_pack_array(void**&,int&)
                (virtual,inline)    ǡѥåμ

        쥯饹 Port ν貾۴ؿǤ.
        ü sender , ǡѥåޤ.
        ǡѥå򻲾ȤȤѤƲ.

        dp_array ˤ, void* ˥㥹Ȥ줿 Data_pack* ѿ
        Ǽ, num_of_dp ˤ dp_array ǿǼޤ.

        㤨, dp_array  k ܤ˼ϿƤǡѥåΥǡ
        Ͽ򻲾Ȥ,
            ((Data_pack*) dp_array[k])->Data_time()
        Τ褦ɬ㥹ȤѤƲ.

        dp_array μǡѥåΥǡϿ郎Ϣ³Ƥ뤳Ȥ,
        Send_port::Stock_data_pack ˤݾڤޤ.

        ɬפʥǡѥåǽݤ, Can_you_get_data ѤƳ
        ǧǤޤ.

        ARGUMENT:
            dp_array      ǡѥåؤΥݥ󥿤Ǽ뻲
                            Ȱ.
            num_of_dp     dp_array ǿǼ뻲Ȱ.
        ACCIDENT:
            ³, sender  NULL ξ.
              δؿ, ׻ˤѤǽΤ, 
               sender  NULL ǤʤȤǧޤǤ.
            sender  Port* ʳѿǼƤ.
        </japanese> */
public:
  //////////////////////////////////////////////////////////////////////
  // Get a data pack at specified time "data_time". The results are
  // stored in "former" and "later".
  //
  // <ul>
  //   <li> "former" is the pointer to the newest data before
  //        "data_time"
  //   <li> "later" is the pointer to the oldest data after
  //        "data_time"
  // </ul>
  //
  // When a data pack just at "data_time" exists, "former" and "later"
  // point to that data pack.
  //
  // "flag" specifies the search option.
  //
  // <dl>
  //  <dt> 'b'
  //  <dd>   Receive both of "former" and "later"
  //
  //  <dt> 'f'
  //  <dd>   Receive "former".  Even if "later" cannot be obtained,
  //         the function returns YES.
  //
  //  <dt> 'l'
  //  <dd>   Receive "later".  Even if "former" cannot be obtained,
  //         the function returns YES.
  //
  //  <dt> other character
  //  <dd>   Receive at lease one of "former" and "later"
  // </dl>
  Boolean Get_data_pack(void*& former, time_t data_time, void*& later,
                          char flag = 'b')
        { return ((Port*) sender)
                    ->Get_data_pack(former, data_time, later, flag); }
        /* <japanese>
          Receive_port::Get_data_pack(void*&,time_t,void*&,char)
                (virtual)   ǡѥåμ

        ꤷ, sender ǡѥåޤ.

        former ˤ, ǡϿ郎 data_time  data_time ˺Ǥ
        ϿǡѥåؤΥݥ󥿤, Ǽޤ.

        later ˤ, ǡϿ郎 data_time ʹߤ data_time ˺Ǥ
        ϿǡѥåؤΥݥ󥿤, Ǽޤ.

        ǡϿ郎 data_time ǤǡѥåϿƤ
        , former, later Ȥ⤽Ǽޤ.

        ǡѥåǤʤ, former, later  NULL
        ꤵޤ.

        flag ϼΥ⡼ɤ򤹤뤿Υץ, ̤Ǥ.
            'b'       former, later ξɬ.
                        Ǥʤ, ξȤ NULL ꤹ.
            'f'       later ǤʤƤ, former Τ߼.
                        later Ǥʤ, NULL ꤹ.
            'l'       former ǤʤƤ, later Τ߼.
                        former Ǥʤ, NULL ꤹ.
            ¾    former, later ΤʤȤ.
                        ǤʤΤ NULL ꤹ.

        flag Υǥեͤ, 'b' Ǥ.
        ޤ,  later Ϥʤ, flag  f Ȥ former Τ
        ,  former Ϥʤ, flag  l Ȥ later 
        ߼ޤ.

        ARGUMENT:
            former        d_time  d_time ˺ǤᤤϿǡѥ
                            ؤΥݥ󥿤Ǽ뻲Ȱ.
            data_time     . (sec)
                            λ˺ǤᤤǡϿļϿ
                            ѥå.
            later         d_time ʹߤ d_time ˺ǤᤤϿǡѥ
                            ؤΥݥ󥿤Ǽ뻲Ȱ.
            flag          ǡѥåμꤹե饰.
        RETURN:
            YES           ΥǡѥåǤ.
            NO            data_pack_array  NULL ξ.
                            ΥǡѥåǤʤ.
        </japanese> */
public:
  //////////////////////////////////////////////////////////////////////
  // Returns the "Necessary_time_from(this)" of the owner of the current
  // receive port.  Because this function is virtual, you can redefine
  // it in derived classes.
  time_t Necessary_time_from(void);

  //////////////////////////////////////////////////////////////////////
  // Returns the "Necessary_time_to(this)" of the owner of the current
  // receive port.  Because this function is virtual, you can redefine
  // it in derived classes.
  time_t Necessary_time_to(void);
        /* <japanese>
          Receive_port::Necessary_time_from(void) (virtual)
          Receive_port::Necessary_time_to(void) (virtual)
                ǡɬ׻֤

        üҤ, Necessary_time_from ʹ Necessary_time_to 
        Υǡѥåõʤ褦˻ؼ뤿ޤ.

        owner  Element::Necessary_time_from, Element::Necessary_time_to
        ֤ޤ. owner  NULL ξ, LONG_MIN, LONG_MAX ֤ޤ.

        ۴ؿʤΤ饹ǺǤޤ.

        RETURN:
            time_t    ǡɬ׻ (sec).
        </japanese> */

  //////////////////////////////////////////////////////////////////////
  // Relay the calculation request to the sender
  void Creq(void) { ((Port*)sender)->Creq(); }

public:
  //////////////////////////////////////////////////////////////////////
  // Remove unnecessary data packs from the memory.
  void Reduce_data_pack_stock(void)
        { ((Port*) sender)->Reduce_data_pack_stock(); }
        /* <japanese>
          Receive_port::Reduce_data_pack_stock(void) (virtual,inline)
                ץǡѥåξõ

        ü sender ץǡѥåõ褦ؼޤ.

        μüҤ owner Ƿ׻λѤ, ץޤ.
        </japanese> */

//  3.  ϵǽ.
public:
  //////////////////////////////////////////////////////////////////////
  // Print the information about the connection to "fp_out" with the
  // string "head"
  void Print_connection(FILE* fp_outr, char* head);
        /* <japanese>
          Receive_port::Print_connection(FILE*,char*) (virtual)
                ³ν

        üҤ³ط˴ؤ fp_out ˽Ϥޤ.
        ۴ؿʤΤ, 饹ǺǤޤ.

        ARGUMENT:
            fp_out    եؤΥݥ. ǥե stderr.
            head      ϻΥإå. ǥեͣ.
        </japanese> */

//; 4.  󥹥ȥ饯, ǥȥ饯.
public:
  //////////////////////////////////////////////////////////////////////
  // Initialization with object name "p_name" and object number "p_num"
  Boolean Init(char* p_name, int p_num)
        { return Port::Init(p_name, p_num); }
        /* <japanese>
          Receive_port::Init(char*,int) (inline)  Ѵؿ

        Port::Init ¹Ԥޤ.

        RETURN:
            YES   .
            NO    Ť˽줿. (Object::Init)
        EXIT:
            5     object_name εΰ褬ݤǤʤ.
                    (Object::Init)
        </japanese> */

  //////////////////////////////////////////////////////////////////////
  // Constructor. Sets object name  to "p_name" and object number to
  // "p_num"
  Receive_port(char* p_name, int p_num);

  //////////////////////////////////////////////////////////////////////
  // Constructor.
  Receive_port(void);

  //////////////////////////////////////////////////////////////////////
  // Destructor.
  ~Receive_port(void);
        /* <japanese>
          Receive_port::Receive_port(char*,int)   󥹥ȥ饯
          Receive_port::Receive_port(void)        󥹥ȥ饯
          Receive_port::~Receive_port(void) (virtual)     ǥȥ饯

        Receive_port(char*,int) , 쥯饹 Port Υǥեȥ󥹥
        饯Ѥ, Receive_port::Init Ѥ, Фʲ̤
        ޤ.
            class_name        : "Receive_port"
            object_name       : p_name (ҡפ˥ԡ)
            object_number     : p_num
            owner             : NULL
            sender            : NULL

        ǥեȥ󥹥ȥ饯, Receive_port::Init ¹Ԥޤ.
        η, ʲȤʤޤ.
            object_name       : NULL
            object_number     : -1
        ¾ΥФФ, Ĥ󥹥ȥ饯ƱǤ.
        ξ, Receive_port::Init ѤƥФƲ.

        ǥȥ饯, ܴ쥯饹 Object  virtual Ƥޤ.

        ARGUMENT:
            p_name    ü̾.
            p_num     üֹ.
        EXIT:
            5         object_name εΰݤǤʤ.
                        (Object::Init)
        </japanese> */

};


#endif  // ! receive_h

/*; Local Variables: *** */
/*; tab-width: 4 *** */
/*; fill-column: 72 *** */
/*; End: *** */

/*; end of file */
