/* osd_tmp.cc {\documentstyle{jsrc2latex}\rhead{\sl osd\_tmp.cc\quad\thepage}
   \font\tenrm=cmtt10}
{\ \begin{verbatim}
$RCSfile: osd_tmp.cc,v $  $Revision: 1.1 $ 
$Date: 2000/12/02 10:07:15 $  by KyotoHydro
\end{verbatim}}
{\ {\setlength{\parskip}{4pt}% latex-mode

The implementation of class OHyMoS_ReceiveStepData<T>.

OHyMoS_ReceiveStepData<T> takes the data as step-formed data. In
step-formed type data, the value is taken as constant equal to the
current data value from the previous data time till the current data
time.

}% end of latex-mode }
*/

#if !defined(sd_tmp_cc)
#define sd_tmp_cc

#include <stdlib.h>
#include <assert.h>
#include "osd_tmp.h"				// definition of class
								// OHyMoS_ReceiveStepData<T>

template <class T>
T OHyMoS_ReceiveStepData<T>::Get_data(time_t t)
{
	// Get data_pack
	void** data_pack_array;
	int number_of_data_packs, ip;

	(*this).Get_data_pack_array(data_pack_array, number_of_data_packs);

	assert(number_of_data_packs > 0);

	time_t ta = ((Data<T>*) data_pack_array[0])->Data_time();
	time_t tb = 
	((Data<T>*) data_pack_array[number_of_data_packs-1])->Data_time();

	// Is\'n there inconsistency in data?
	assert(t >= ta && t <= tb);

	// Find the first data time later than t
	time_t tn; T yn;			// next data_pack
	for (ip = 0; ip < number_of_data_packs; ip++) {
		tn = ((Data<T>*) data_pack_array[ip])->Data_time();
		yn = ((Data<T>*) data_pack_array[ip])->Value();
		if (tn > t) break;
	}

	return yn;
}

template <class T>
T OHyMoS_ReceiveStepData<T>::Get_data_sum(time_t t1, time_t t2)
{


	// Is\'nt there inconsistency in data?
	assert(t1 > t2);

	T sum = 0;
	if (t1 == t2) { return sum; }

	// Get data_pack
	void** data_pack_array;
	int number_of_data_packs, ip;

	(*this).Get_data_pack_array(data_pack_array, number_of_data_packs);

	assert(number_of_data_packs > 0);

	time_t ta = ((Data<T>*) data_pack_array[0])->Data_time();
	time_t tb = 
	((Data<T>*) data_pack_array[number_of_data_packs-1])->Data_time();

	// Is\'n there inconsistency in data?
	assert(t1 >= ta && t2 <= tb);

	// Find the first data time later than t1
	time_t ts = t1;
	time_t tn; T yn;			// next data_pack
	for (ip = 0; ip < number_of_data_packs; ip++) {
		tn = ((Data<T>*) data_pack_array[ip])->Data_time();
		yn = ((Data<T>*) data_pack_array[ip])->Value();
		if (tn > t1) break;
	}

	// Add the bar part before t2
	while (tn <= t2) {
		sum += yn * (tn - ts);
		ts = tn;   ip++;
		tn = ((Data<T>*) data_pack_array[ip])->Data_time();
		yn = ((Data<T>*) data_pack_array[ip])->Value();
	}
	
	// Add the remaining bar untill time t2 
	if (t2 < tn) {
		sum += yn * (t2 - ts);
	}

	/* Output of data */
	return sum;
}

// added by Y.Ichikawa (Wed Feb 14 21:55:10 1996)
// -------------------------------------------------------------
// instantiation
// -------------------------------------------------------------

template class OHyMoS_ReceiveStepData<double>;
template class OHyMoS_ReceiveStepData<float>;
template class OHyMoS_ReceiveStepData<int>;
template class OHyMoS_ReceiveStepData<long>;


#endif // ! sd_tmp_cc
 
/* end of file {\ % The following part will be omitted in tex file
\begin{comment}\begin{verbatim}
This part is not included in the tex output.
$Revision: 1.1 $

# Local Variables: ***
# tab-width: 4 ***
# End: ***
\end{verbatim}\end{comment}
} */

